/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Nexus.clm.SettingsView', {
    extend: 'Ext.Panel',

    requires: [
      'Nexus.staging.repository.activity.EventDetail',
      'Nexus.staging.repository.SummaryView',
      'Nexus.staging.repository.RepositoriesMediator'
    ],

    requirejs: [
      'nexus/siesta',
      'Nexus/clm'
    ],

    mixins: [
      'Nexus.LogAwareMixin'
    ],

    constructor: function (config) {
        var self = this;

        self.clmServerURL = new Ext.form.TextField({
            name: 'clmServerURL',
            fieldLabel: 'CLM Server URL',
            helpText: 'The address of your CLM Server',
            anchor : Sonatype.view.FIELD_OFFSET,
            allowBlank : false,
            itemCls : 'required-field',
            validator : function(v) {
              if (v === '' || v.match(/^(?:http|https):\/\//i))
              {
                return true;
              }
              else
              {
                return 'Protocol must be http:// or https://';
              }
            }
        });


        self.settingsForm = new Ext.FormPanel({
          border : false,
          labelWidth : 100,
          layoutConfig : {
            labelSeparator : ''
          },
            items: [
                self.clmServerURL,
              {
                xtype: 'textarea',
                fieldLabel: 'Properties',
                name: 'clmConfiguration',
                anchor : Sonatype.view.FIELD_OFFSET,
                helpText: 'Additional properties to configure for the CLM Server',
                flex: 1
              }
            ],
            buttons : [
              {
                text : 'Save',
                tooltip : 'Save settings',
                formBind : true,
                scope : self,
                handler : self.saveSettings
              },
              { text : 'Reset',
                tooltip : 'Reset changes',
                formBind : true,
                scope : this,
                handler : self.loadSettings
              }
            ]
        });

      Ext.apply(self, {
          cls: 'nx-clm-SettingsView',
          border: false,
          layout: 'vbox',
          layoutConfig: {
              align: 'stretch'
          },
          items: [
              {
                  xtype: 'fieldset',
                  title: 'Settings',
                  anchor : Sonatype.view.FIELDSET_OFFSET,
                  autoHeight : true,
                  items: [
                    {
                      border : false,
                      html : '<div class="nx-clm-doco">'
                            +   '<a href="http://links.sonatype.com/products/clm/nexus/home"  target="_blank">'
                            +   'Make Nexus Staging even more powerful with Sonatype CLM'
                            +   ' </a>'
                            + '</div>'
                    },
                      self.settingsForm
                  ]
              }
          ]
      });

      self.constructor.superclass.constructor.apply(self, arguments);

      // Load settings after layout, has to be done post superclass.constructor()
      self.on('afterlayout', self.loadSettings, self, {
        single: true
      });

    },

    saveSettings : function() {
      var self = this, form;

      self.logDebug('CLM Saving settings');

      form = self.settingsForm.getForm();

      if(!form.isValid())
      {
        return;
      }

      Ext.Ajax.request({
        url : Nexus.clm.resourcePath,
        method : 'POST',
        scope : self,
        suppressStatus: 400,
        jsonData : {
          clmServerURL : form.findField('clmServerURL').getValue(),
          clmConfiguration: form.findField('clmConfiguration').getValue()
        },

        success : function() {
          Nexus.clm.showMessage('Configuration saved');
          form.items.each(function(item){
            item.clearInvalid();
          });
        },

        failure: function(response, opts){
          if(response.siestaValidationError !== undefined)
          {
            Ext.each(response.siestaValidationError, function (error){
              form.findField(error.id).markInvalid(error.message);
            });
          }
        }
      });
    },

    loadSettings : function() {
      var self = this;

      self.logDebug('CLM Loading settings');

      Ext.Ajax.request({
        url :  Nexus.clm.resourcePath,
        method : 'GET',
        scope : self,
        success : function(response) {
          self.settingsForm.getForm().setValues(Ext.decode(response.responseText));
          self.logDebug('CLM Settings loaded');
        }
      });
    }
},

function () {
    var type = this;
    var key = Nexus.clm.STAGING_KEY;
    var label = Nexus.clm.STAGING_LABEL;

    // Attach SettingsView to navigation panel if current user has appropriate permissions
    Sonatype.Events.addListener('nexusNavigationInit', function (nexusPanel) {
      var p = Sonatype.lib.Permissions;
      if (p.checkPermission('nexus:settings', p.READ)) {
        nexusPanel.add({
          enabled: true,
          sectionId: 'st-nexus-config',
          title: Nexus.clm.TITLE,
          tabId: Nexus.clm.TAB_ID,
          tabTitle: Nexus.clm.TAB_TITLE,
          tabCode: type
        });
      }
    });

  //Enhance StagingProfile UI model with CLM metadata. Works in conjunction with the staging.profile.submitted event.
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.profile.loaded', function(items, properties, dataRoot) {
    NX.log.debug("Enhancing UI with CLM details.");
    items.push(new Ext.form.TextField({
      fieldLabel : label,
      name : dataRoot + '.' + key,
      value : properties[key],
      width : 376,
      helpText : 'Enter the CLM Application Id here to enable policy enforcement for this profile.<br/>'
            + '<a href="http://links.sonatype.com/products/clm/nexus/home"  target="_blank">'
            + 'Make Nexus Staging even more powerful with Sonatype CLM'
            + ' </a>',
      persistent : true
    })
    );
  });

  //Enhance StagingProfile UI model to serialize CLM metadata. Works in conjunction with the staging.profile.loaded event.
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.profile.submitted', function(referenceData) {
    NX.log.debug("Enhancing StagingProfile submit data with CLM details.");
    referenceData[key] = null;
  });

  /**
   * @param value The encoded value with details of last CLM result.
   * @param showDetailedReport Whether or not a link to the full CLM results should be included
   * @return {*} view of the last CLM result.
   */
  function renderClmValues(value, showDetailedReport) {
    var values = value.split(',');
    NX.assert(values.length == 6, "Should have 6 comma separated values stored for CLM violations.");
    return Nexus.clm.chickletTemplate.apply({
      criticalCount : values[2],
      severeCount : values[3],
      moderateCount : values[4],
      detailedReport : showDetailedReport ? values[5] : null
    });
  }

  // enhance StagingRepository grid view with an additional column
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.repositoriesGrid.configureColumns', function(columns) {
    NX.log.debug("Enhancing Repository grid view with CLM details.");
    columns.push({
      id : 'violations',
      width : 250,
      header : 'Violations',
      renderer : function(value, metaData, record) {
        var properties = record.data.properties;
        if (properties !== undefined) {
          var clmProperty;
          Ext.each(properties, function(property) {
            if (property.name == "Violations") {
              clmProperty = property.value;
              return false;
            }
          });
          if(clmProperty !== undefined)
          {
            return renderClmValues(clmProperty, false);
          }
        }
        return 'No CLM data recorded';
      }
    });
  });

  //Register renderer for CLM summary in StagingRepository summary view.
  Nexus.staging.repository.RepositoriesMediator.addListener('staging.repositorySummary.configureProperties', function(properties, values){
    Ext.each(values.properties, function(property){
      if(property.name && property.value && property.name === 'Violations')
      {
        NX.log.debug("Enhancing Repository summary view with CLM details.");
        for(var i = 0; i <  properties.length; i++)
        {
          if(properties[i].name === 'Activity')
          {
            break;
          }
        }
        properties.splice(i+1, 0, { name: 'Violations', value: renderClmValues(property.value, true) });
        return false;
      }
    });
  });

  //Register renderers for CLM rules in the Staging UI
  Nexus.staging.repository.activity.EventDetail.registerBodyTemplateFactory('ruleFailed', Nexus.clm.stagingRule);
  Nexus.staging.repository.activity.EventDetail.registerBodyTemplateFactory('rulePassed', Nexus.clm.stagingRule);
});
